/*
    *
    * This file is a part of CoreStats.
    * A real time system resource viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/

#include <QToolButton>
#include <QScrollArea>

#include <cprime/themefunc.h>

#include "pbattery.h"
#include "pdisplay.h"
#include "pdrives.h"
#include "pgeneral.h"
#include "presources.h"

#include "corestats.h"
#include "ui_corestats.h"


corestats::corestats(QWidget *parent): QWidget(parent), ui(new Ui::corestats)
{
    ui->setupUi(this);

    loadSettings();
    startSetup();

    // Hide the battery button if there is no battery
    if (not BatteryManager::instance()->batteries().count()) {
        ui->Bbattery->setVisible(false);
    }

    on_Bgeneral_clicked();
    on_appTitle_clicked();
}

corestats::~corestats()
{
    delete ui->pages;
    delete ui;
}

/**
 * @brief Setup ui elements
 */
void corestats::startSetup()
{
    // set stylesheet from style.qrc
    ui->sideView->setStyleSheet(CPrime::ThemeFunc::getSideViewStyleSheet());
    ui->sideView->setFixedWidth(sideViewSize);

    if (touch) {
        this->setWindowState(Qt::WindowMaximized);
    } else {
        this->resize(800, 500);
    }

    // all toolbuttons icon size in sideView
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        if (b) {
            b->setIconSize(sideViewIconSize);
        }
    }
}

/**
 * @brief Loads application settings
 */
void corestats::loadSettings()
{
    // Sideview is visible in default
    sideView = 1;

    sideViewIconSize = sm->value("CoreApps", "SideViewIconSize");
    listViewIconSize = sm->value("CoreApps", "ListViewIconSize");
    sideViewSize = sm->value("CoreApps", "SideViewSize");
    touch = sm->value("CoreApps", "TouchMode");
}

void corestats::closeEvent(QCloseEvent *event)
{
    event->ignore();

    for (int i = 0; i < ui->pages->count(); i++) {
        if (ui->pages->widget(i)->children().count() == 2)
            if (ui->pages->widget(i)->children().at(1)) {
                ui->pages->widget(i)->children().at(1)->deleteLater();
            }
    }

    event->accept();
}

void corestats::pageClick(QToolButton *btn, int i, QString title)
{
    // all button checked false
    for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
        b->setChecked(false);
    }

    btn->setChecked(true);
    ui->selectedsection->setText(title);
    ui->pages->setCurrentIndex(i);
    this->setWindowTitle(title + " - CoreStats");
}

void corestats::on_Bgeneral_clicked()
{
    if (ui->pages->widget(0)->children().count() < 2) {
        pgeneral *infoo = new pgeneral(this);
        ui->pGeneralLayout->addWidget(infoo);
    }

    pageClick(ui->Bgeneral, 0, tr("General"));
}

void corestats::on_Bdrives_clicked()
{
    if (ui->pages->widget(1)->children().count() < 2) {
        pDrives *drives = new pDrives(this);
        ui->pDriveLayout->addWidget(drives);
    }

    pageClick(ui->Bdrives, 1, tr("Drives"));
}

void corestats::on_Bbattery_clicked()
{
    if (ui->pages->widget(2)->children().count() < 2) {
        pBattery *battery = new pBattery(this);
        ui->pBatteryLayout->addWidget(battery);
    }

    pageClick(ui->Bbattery, 2, tr("Battery"));
}

void corestats::on_Bresource_clicked()
{
    if (ui->pages->widget(3)->children().count() < 2) {
        presources *resourcePage = new presources(this);
        QScrollArea *base = new QScrollArea(this);
        base->setFrameShape(QFrame::NoFrame);
        base->setWidgetResizable(true);
        base->setWidget(resourcePage);
        ui->pResourceLayout->addWidget(base);
    }

    pageClick(ui->Bresource, 3, tr("Resource"));
}

void corestats::on_Bdisplay_clicked()
{
    if (ui->pages->widget(4)->children().count() < 2) {
        pDisplay *display = new pDisplay(this);
        ui->pDisplayLayout->addWidget(display);
    }

    pageClick(ui->Bdisplay, 4, tr("Display"));
}


void corestats::on_appTitle_clicked()
{
    if (sideView && touch) {
        // all button without text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonIconOnly);
            b->setFixedWidth(sideViewIconSize.width() + 10);
        }

        ui->sideView->setFixedWidth(sideViewIconSize.width() + 10);
        sideView = 0;
    } else {
        // all button with text
        for (QToolButton *b : ui->sideView->findChildren<QToolButton *>()) {
            b->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
            b->setFixedWidth(sideViewSize);
        }

        ui->sideView->setFixedWidth(sideViewSize);
        sideView = 1;
    }

}
