/*
    *
    * This file is a part of CoreStats.
    * A real time system resource viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/


#include <QStringListModel>
#include <QDesktopWidget>
#include <QDebug>

#include <cprime/cprime.h>
#include <csys/system_info.h>

#include "usagebar.h"
#include "pgeneral.h"
#include "ui_pgeneral.h"


pgeneral::pgeneral(QWidget *parent) : QWidget(parent), ui(new Ui::pgeneral),
	timer(new QTimer(this)),
	im(InfoManager::ins())
{
	ui->setupUi(this);

	init();
	systemInformationInit();
}

pgeneral::~pgeneral()
{
	delete ui;
}

void pgeneral::init()
{
	// Resource bars
    cpuBar = new UsageBar("CPU", im->getCpuCoreCount(), this);
    cpuBar->setMax(100);
    cpuBar->setUnits(UsageBar::PERCENT);

    memBar = new UsageBar("Memory", 2, this);
	im->updateMemoryInfo();
    memBar->setMax(qMax(im->getSwapTotal(), im->getMemTotal()));
    memBar->setUnits(UsageBar::SIZE);

    netBar = new UsageBar("Network", 2, this);
    netBar->setUnits(UsageBar::SPEED);

    ui->barsLyt->addWidget( cpuBar );
    ui->barsLyt->addWidget( memBar );
    ui->barsLyt->addWidget( netBar );

	// connections
	connect(timer, &QTimer::timeout, this, &pgeneral::updateCpuBar);
	connect(timer, &QTimer::timeout, this, &pgeneral::updateMemoryBar);
	connect(timer, &QTimer::timeout, this, &pgeneral::updateNetworkBar);

	// 1 second timer
    timer->start(1000);

	// initialization
	updateCpuBar();
	updateMemoryBar();
	updateNetworkBar();

	ui->scrollLyt->addStretch();
}

void pgeneral::systemInformationInit()
{
	QDesktopWidget dw;

    QString scr = QString( "%1 x %2" ).arg( dw.width() ).arg( dw.height() );

	// get system information
	SystemInfo sysInfo;

	QStringList left;
	left << "User Name" << "Platform" << "Distribution" << "Kernel Release"
         << "CPU Model" << "CPU Speed" << "CPU Cores" << "Display Size"
         << "Bootup time" << "Uptime" << "Qt Toolkit Version";

	QStringList right;
    right << sysInfo.getUsername() << sysInfo.getPlatform() << sysInfo.getDistribution()
          << sysInfo.getKernel() << sysInfo.getCpuModel() << sysInfo.getCpuSpeed()
          << sysInfo.getCpuCore() << scr << sysInfo.getBootTime() << sysInfo.getUptime() << QT_VERSION_STR;

    ui->infoLeft->clear();
    ui->infoRight->clear();

    int count = left.count();
    QString sep = "\n";
    for (int i = 0; i < count; i++) {
        if (i + 1 == count) {
            sep = "";
        }

        ui->infoLeft->setText(ui->infoLeft->text() + left[i] + sep);
        ui->infoRight->setText(ui->infoRight->text() + ": " + right[i] + sep);
    }
}

void pgeneral::updateCpuBar()
{
	ResourceValues cpuValues;
	int cpuUsage = 0;
	int cpuCount = im->getCpuCoreCount();

    Q_FOREACH (int i, im->getCpuPercents()) {
        cpuValues << static_cast<unsigned long>(i);
		cpuUsage += i;
	}

    if (cpuValues.count() == cpuCount + 1) {
		cpuValues.removeFirst();
    }

    cpuBar->setValues(cpuValues);
}

void pgeneral::updateMemoryBar()
{
	im->updateMemoryInfo();

	ResourceValues memValues;

    if (im->getMemTotal()) {
		memValues << im->getMemUsed();
    }

    else {
		memValues << 0;
    }

    if (im->getSwapTotal()) {
		memValues << im->getSwapUsed();
    }

    else {
		memValues << 0;
    }

    memBar->setValues(memValues);
}

void pgeneral::updateNetworkBar()
{
	quint64 RecvBytes = im->getRXbytes();
	quint64 TrnsBytes = im->getTXbytes();

	quint64 RecvSpeed = prevRXBytes ? RecvBytes - prevRXBytes : 0;
	quint64 TrnsSpeed = prevTXBytes ? TrnsBytes - prevTXBytes : 0;

	prevRXBytes = RecvBytes;
	prevTXBytes = TrnsBytes;

    quint64 max = CPrime::FileFunc::maxYForSize(qMax(RecvSpeed, TrnsSpeed));
	max = max > 1 ? max : 1;

	ResourceValues dataValues = ResourceValues() << RecvSpeed << TrnsSpeed;
    // ResourceValues dataValues = ResourceValues() << 1 * max << 1 * max;          // For testing purposes
    netBar->setMax(max);
    netBar->setValues(dataValues);
}

void pgeneral::resizeEvent( QResizeEvent *rEvent ) {

//	QSize newSize = rEvent->size();
//	int newWidth = newSize.width() / 3;

//	if ( cpuBar->minimumWidth() < newWidth - 5 )
//		cpuBar->setFixedWidth( newWidth - 5 );

//	if ( memBar->minimumWidth() < newWidth - 5 )
//		memBar->setFixedWidth( newWidth - 5 );

//	if ( netBar->minimumWidth() < newWidth - 5 )
//		netBar->setFixedWidth( newWidth - 5 );

    QSize newSize = rEvent->size()/3;

    qDebug () << newSize<<rEvent->size() ;

    cpuBar->resize(newSize);
    memBar->resize(newSize);
    netBar->resize(newSize);
};
