/*
    *
    * This file is a part of CoreStats.
    * A real time system resource viewer for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, write to the Free Software
    * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    * MA 02110-1301, USA.
    *
*/


#include <cprime/cprime.h>

#include "usagebar.h"


const int BAR_WIDTH = 70;
const int TEXT_HEIGHT = 15;

static inline QString formatSize(quint64 fileSize)
{
	QList<quint64> sizes = { 1ULL << 10, 1ULL << 20, 1ULL << 30, 1ULL << 40 };

    QString sizeStr = QString::number(fileSize);

    Q_FOREACH (quint64 size, sizes) {
        if (fileSize < size) {
            sizeStr = QString::number(1.0 * fileSize / (size >> 10), 'f', (size >> 10 > 1 ? 1 : 0));
			break;
		}
	}

    return sizeStr + " " + CPrime::FileFunc::formatSizeStr(fileSize);
}

UsageBar::UsageBar(QString resource, int segments, QWidget *parent) : QWidget(parent)
{
	mSegments = segments;
	mUnits = UsageBar::PERCENT;
    resName = QString(resource);

    minWidth = mSegments * (BAR_WIDTH + 1) - 1;
    setMinimumWidth( static_cast<int>( minWidth ) );

    setFixedHeight(200);

    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);

    for (int i = 0; i < segments; i++) {
        mValues.append(0);
    }
}

void UsageBar::setMax(quint64 max)
{
	mMax = max;

	repaint();
}

void UsageBar::setUnits(UsageBar::Units units) {
	mUnits = units;

    if (mUnits == UsageBar::PERCENT) {
        minWidth = mSegments * (BAR_WIDTH + 1) * 0.5 - 1;
    }

    else if (mUnits == UsageBar::SIZE) {
        minWidth = mSegments * (BAR_WIDTH + 1) * 0.7 - 1;
    }

    else {
        minWidth = mSegments * (BAR_WIDTH + 1) - 1;
    }

    setMinimumWidth( minWidth );

	repaint();
}

void UsageBar::setValues(ResourceValues values)
{
	mValues.clear();
	mValues << values;

	repaint();
}

QSize UsageBar::sizeHint()
{
    return QSize( minWidth, height() );
}

QSize UsageBar::minimumSizeHint()
{
	return sizeHint();
}

int UsageBar::minimumWidth()
{
	return minWidth;
}

void UsageBar::paintEvent(QPaintEvent *pEvent)
{
    QPainter painter(this);
    painter.fillRect(rect(), Qt::transparent);

    QPalette pltt(qApp->palette());

    QColor pen(pltt.color(QPalette::Highlight));
    QColor brush(pen);
    brush.setAlpha(90);

    QColor usage(pen);
    QColor danger = QColor(0x8B, 0x00, 0x00);

	quint64 h = height() - TEXT_HEIGHT * 3;
    quint64 w = (width() - minWidth) / 2;

    for (int s = 0; s < mSegments; s++) {
		painter.save();

        painter.setPen(Qt::NoPen);
        painter.setBrush(usage);

        double u = mValues.value(s) * (h - TEXT_HEIGHT) / mMax;

        if (u > h) {
            painter.setBrush(danger);
        }

		int x = 0, bWidth = BAR_WIDTH;
		QString text;

        if (mUnits == UsageBar::PERCENT) {
            x = static_cast<int>(w + s * (BAR_WIDTH * 0.5 + 1));
            text = QString::number(mValues.value(s)) + " %";
            bWidth = static_cast<int>(BAR_WIDTH * 0.5);
        } else if (mUnits == UsageBar::SIZE) {
            x = static_cast<int>(w + s * (BAR_WIDTH * 0.7 + 1));
            text = formatSize(mValues.value(s));
            bWidth = static_cast<int>(BAR_WIDTH * 0.7);
        } else if (mUnits == UsageBar::SPEED) {
            x = w + s * (BAR_WIDTH + 1);
            text = formatSize(mValues.value(s)) + "/s";
			bWidth = BAR_WIDTH;
		}

        painter.drawRect(QRectF(x, (h - u + TEXT_HEIGHT), bWidth, u));
		painter.restore();
		painter.save();
        painter.setPen(pen);
        painter.setFont(QFont(font().family(), 8));
        painter.drawText(QRectF(x, (h - u), bWidth, TEXT_HEIGHT), Qt::AlignCenter, text);
		painter.restore();
	}

    painter.setPen(palette().color(QPalette::Text));
    painter.setFont(qApp->font());
    painter.drawText(QRect(0, h + TEXT_HEIGHT, width(), 2 * TEXT_HEIGHT), Qt::AlignCenter, resName);
	painter.end();
	pEvent->accept();
}
